%% |estimateFAVAR.m| -- Forecast 113 series with a FAVAR

%% Clear the workspace and load the data

clear();
close('all');
format('compact');
d = loadstruct('data.mat');
list = fieldnames(d);

%% Display the list of series
% Each tseries object (in fact, each model, VAR. FAVAR) can have 'user
% data' attached to it. The user data can be absolutely anything.

% Display user data for the 1st variable (whatever it is).
u = userdata(d.(list{1}));
display(u);

% Display the whole list of variables.
nlist = numel(list);
for i = 1 : nlist
   u = userdata(d.(list{i}));
   fprintf('%3g. %8s %g\t\t%s\n',i,list{i},u.Transform,u.Descriptor);
end

%% Define dates

startHist = mm(1991,1);
endHist = mm(2009,10);
histRange = startHist : endHist;

%% Check the jagged edge
% Get the observations for the last 12 months, and display what's available
% and what's missing.

Y = db2array(d,list,endHist-11:endHist);
spy(~isnan(Y));

%% Estimate a FAVAR
% The state-space form of a FAVAR is as follows:
%
% <latex>
% \begin{gather*}
% y_t = C x_t + \overline y + u_t, \\
% x_t = A x_{t-1} + B e_t \\
% E u_t u_t' = \Sigma, \quad E e_t e_t' = \Omega
% \end{gather*}
% </latex>
%
% The estimation step, using the |estimate| function, only exploits
% information in periods where there are all observations available. The
% factor output series might, therefore, have gaps in them.
%
% The output arguments of the |estimate| function are
%
% * |a| -- an estimated FAVAR object,
% * |d1| -- a database with the original input series,
% * |c1| -- a database with the common components
% * |x1| -- a multivariate time series of the estimated factors (with
% potentially some NaN whenever observations are not available).
% * |u1| -- a mutlivariate time series of the idiosyncratic shocks,
% * |e1| -- a multivariate time series of the factor shocks.

% * order -- is the number of lags in VAR

% Inputs: estimate(f,d,list,range,[x,y],'order',p)
% x is the target share of the variance explained by the factors
% y is the maximum number of factors that will be used to achieve x
% p is the lag order of the VAR in the factors

a = FAVAR();
[a,d1,c1,x1,u1,e1] = estimate(a,d,list,histRange,[1,4],'order',1);
display(a);

%% How much the FAVAR explains
% Plot cum sum of singular values.

s = get(a,'singval');
s = cumsum(s);
figure();
plot(s/s(end),'lineWidth',1.5,'marker','*');
grid('on');
title('Cumulative normalised singular values');

%% Get the state-space form.

% Get the state-space matrices.
A = get(a,'A*');
B = get(a,'B');
C = get(a,'C');
Omg = get(a,'Omega');
Sgm = get(a,'Sigma');

%% Re-filter the historical data
% Use the |filter| function just to run a Kalman filter on the historical
% data, taking the estimated parameters as given, and fill in the missing
% observations.

[a2,d2,c2,x2,u2,e2] = filter(a,d1,startHist:endHist);


% Compare the missing observations in d1 and d2.
figure();
subplot(2,1,1);
Y1 = db2array(d1,list,endHist-11:endHist);
spy(~isnan(Y1));
subplot(2,1,2);
Y2 = db2array(d2.mean,list,endHist-11:endHist);
spy(~isnan(Y2));

%% Forecast with a FAVAR
% To produce a forecast, you can either extend the |filter| command beyond
% the last historical observation, or use the |forecast| function based on
% the |filter| results. These two yield identical results.

[a3,d3,c3,x3,u3,e3] = filter(a,d,startHist:endHist+40);

[a4,d4,c4,x4,u4,e4] = filter(a,d,histRange);
[d5,c5,x5,u5,e5] = forecast(a,x4,endHist+(1:40),d);

% Plot the factors.
nx = size(x3.mean,2);
figure();
for i = 1 : nx
   subplot(2,2,i);
   h = plot([ ...
      x3.mean{:,i}, ...
      x3.mean{:,i} + x3.std{:,i}, ...
      x3.mean{:,i} - x3.std{:,i}, ...
      ],'dateformat','YY:MM','lineWidth',1.5);
   set(h(2:3), ...
      'color',get(h(1),'color'), ...
      'lineStyle', ':');
   title(sprintf('Factor #%g',i));
   grid('on');
   highlight(endHist+(1:40));
   zeroline();
end

